from .agis import AGISSample as AGISSample
from .csv_config import ColumnStrEnum as ColumnStrEnum
from .csv_config import CSVConfig as CSVConfig
from .csv_config import encodingStr as encodingStr
from .dma import DMASample as DMASample
from .instron import InstronSample as InstronSample
from .models import Base as Base
from .models import Sample as Sample



from typing import Any

import pandas as pd

from .csv_config import ColumnStrEnum as col
from .csv_config import CSVConfig
from .models import Sample


class AGISSample(Sample):
    calibration_coefficient: float = 0.84  # 2023/1/02時点
    width_mm: float = 0
    length_mm: float = 0
    thickness_μm: float = 0
    mean_range: int = 100

    def model_post_init(
        self, __context: Any
    ) -> None:  # インスタンス生成後に実行される。csvから試料の大きさを取得する
        super().model_post_init(__context)
        if not all([self.width_mm, self.length_mm, self.thickness_μm]):
            self.set_sample_size()
        return

    def set_sample_size(self) -> None:
        size_df = pd.read_csv(
            self.file_path,
            **CSVConfig(
                skiprows=[n for n in range(10)],
                nrows=1,
                usecols=[1, 2, 3],
                dtype={"1": float, "2": float, "3": float},
            ).to_dict(),
        )

        thickness_mm, self.width_mm, self.length_mm = size_df.values[0]
        self.thickness_μm = thickness_mm * 1000
        return

    def trim_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        roll = pd.DataFrame(df[col.Force].rolling(window=self.mean_range).mean().diff())

        start = (
            int(roll[col.Force][self.mean_range : self.mean_range * 2].idxmax())
            - self.mean_range
            + 1
        )  # 傾きが最大のところを探す

        result = df[start:].reset_index(drop=True)
        result[col.Stroke] = result[col.Stroke] - result[col.Stroke][0]
        result[col.Force] = result[col.Force] - result[col.Force][0]
        return result

    def calc_stress_strain_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        area_mm2 = self.width_mm * self.thickness_μm / 1000

        stress_Mpa = self.calibration_coefficient * df[col.Force] / area_mm2
        strain = df[col.Stroke] / self.length_mm

        return pd.DataFrame(
            {col.Strain: strain, col.Stress: stress_Mpa},
        )

    def get_stress_strain_df(self) -> pd.DataFrame:
        df = pd.read_csv(self.file_path, **CSVConfig().AGIS().to_dict())
        return self.calc_stress_strain_df(self.trim_df(df))



import pandas as pd
import pytest
from process_nitta.csv_config import CSVConfig


class TestInstronCSVConfig:
    def test_csv_config_instron_正常系(self):
        csv_config = CSVConfig().Instron().to_dict()
        pd.read_csv("./process-nitta/test/csv/instron.csv", **csv_config)

    def test_csv_config_instron_異常系_ファイルパスが存在しない(self):
        csv_config = CSVConfig().Instron().to_dict()

        with pytest.raises(FileNotFoundError):
            pd.read_csv("invalid_path", **csv_config)


class TestAgisCSVConfig:
    def test_csv_config_agis_正常系(self):
        csv_config = CSVConfig().AGIS().to_dict()
        pd.read_csv("./process-nitta/test/csv/agis.csv", **csv_config)

    def test_csv_config_agis_異常系_ファイルパスが存在しない(self):
        csv_config = CSVConfig().AGIS().to_dict()

        with pytest.raises(FileNotFoundError):
            pd.read_csv("invalid_path", **csv_config)


class TestDmaCSVConfig:
    def test_csv_config_dma_正常系(self):
        csv_config = CSVConfig().DMA().to_dict()
        pd.read_csv("./process-nitta/test/csv/dma.csv", **csv_config)

    def test_csv_config_dma_異常系_ファイルパスが存在しない(self):
        csv_config = CSVConfig().DMA().to_dict()

        with pytest.raises(FileNotFoundError):
            pd.read_csv("invalid_path", **csv_config)


class TestIrCSVConfig:
    def test_csv_config_ir_正常系(self):
        csv_config = CSVConfig().IR().to_dict()
        pd.read_csv("./process-nitta/test/csv/ir.csv", **csv_config)

    def test_csv_config_ir_異常系_ファイルパスが存在しない(self):
        csv_config = CSVConfig().IR().to_dict()

        with pytest.raises(FileNotFoundError):
            pd.read_csv("invalid_path", **csv_config)



import pandas as pd

from .csv_config import ColumnStrEnum as col
from .csv_config import CSVConfig
from .models import Sample


class InstronSample(Sample):
    speed_mm_per_min: float
    freq_Hz: float = 0.05
    load_cell_max_N: int = 100
    load_cell_calibration_coef: float = 1
    max_Voltage: float = 10
    mean_range: int = 100

    def trim_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        roll = pd.DataFrame(
            df[col.Voltage].rolling(window=self.mean_range).mean().diff()
        )

        start = (
            int(roll[col.Voltage][self.mean_range : self.mean_range * 2].idxmax())
            - self.mean_range
            + 1
        )  # 傾きが最大のところを探す
        end = int(roll[col.Voltage].idxmin()) + 10

        result = df[start:end].reset_index(drop=True)
        result[col.Voltage] = (
            result[col.Voltage] - result[col.Voltage][0]
        )  # 初期値を0にする

        return result

    def calc_stress_strain_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        area_mm2 = self.width_mm * self.thickness_μm / 1000
        speed_mm_per_sec = self.speed_mm_per_min / 60

        stress_Mpa = (
            self.load_cell_max_N
            / (self.load_cell_calibration_coef * self.max_Voltage)
            / area_mm2
            * df[col.Voltage]
        )
        strain = speed_mm_per_sec * self.freq_Hz * df.index / self.length_mm

        return pd.DataFrame(
            {col.Strain: strain, col.Stress: stress_Mpa},
        )

    def get_stress_strain_df(self) -> pd.DataFrame:
        df = pd.read_csv(self.file_path, **CSVConfig().Instron().to_dict())
        return self.calc_stress_strain_df(self.trim_df(df))



from enum import StrEnum
from typing import Dict, List, Optional, Union

from pydantic import BaseModel


class encodingStr(StrEnum):
    shift_jis = "shift-jis"
    utf_8 = "utf-8"


class ColumnStrEnum(StrEnum):
    Voltage = "Voltage"
    Force = "Force /N"
    Stroke = "Stroke /mm"
    Wave_number = "Wave number /cm$^{-1}$"
    Absorbance = "Absorbance /a.u."
    Strain = "Strain $\epsilon$ /-"  # type: ignore
    Stress = "Stress $\sigma$ /MPa"  # type: ignore
    Temperature = "Temperature /℃"
    E1 = "$\it E'$ /Pa"  # type: ignore
    E2 = "$\it E''$ /Pa"  # type: ignore
    TanDelta = "tan $\delta$"  # type: ignore


col = ColumnStrEnum


class CSVConfig(BaseModel):
    encoding: encodingStr = encodingStr.shift_jis
    sep: str = ","
    header: Optional[int] = None
    usecols: Union[List[int], List[str], None] = None
    names: Optional[List[str]] = None
    dtype: Optional[Dict[str, type]] = None
    skiprows: Optional[List[int]] = None  # 冒頭の行を読み飛ばす動作は許可しない
    skipfooter: int = 0
    engine: str = "python"
    nrows: Optional[int] = None

    def to_dict(self) -> Dict:
        return self.model_dump()

    def Instron(self) -> "CSVConfig":
        self.header = 51
        self.skipfooter = 3
        self.usecols = [col.Voltage]
        self.names = ["EndHeader", "日時(μs)", col.Voltage]
        self.dtype = {col.Voltage: float}
        return self

    def AGIS(self) -> "CSVConfig":
        self.header = 19
        self.usecols = [col.Force, col.Stroke]
        self.names = ["sec", col.Force, col.Stroke]
        self.dtype = {col.Force: float, col.Stroke: float}
        return self

    def DMA(self) -> "CSVConfig":
        self.header = 27
        self.skiprows = [28]
        self.usecols = [
            col.Temperature,
            col.E1,
            col.E2,
            col.TanDelta,
        ]
        self.names = [
            "TOTAL",
            "BL",
            "No",
            col.Temperature,
            "FREQ.",
            "ω",
            col.E1,
            col.E2,
            "E*",
            col.TanDelta,
            "η'",
            "η''",
            "η * ",
            "Time",
            "DISP",
            "DISP.1",
            "FORCE",
            "LOAD",
            "C.D",
            "PHASE",
            "応力",
            "S.STRESS",
            "N.STRESS",
            "REVL",
            "HUMIDITY ",
            "F/L",
            "Ld",
            "TempB",
            "TempC",
            "J*",
            "J'",
            "J''",
            " W ",
            " TempC",
            "*",
        ]
        self.dtype = {
            col.Temperature: float,
            col.E1: float,
            col.E2: float,
            col.TanDelta: float,
        }
        return self

    def IR(self) -> "CSVConfig":
        self.header = None
        self.usecols = [col.Wave_number, col.Absorbance]
        self.names = [col.Wave_number, col.Absorbance]
        self.dtype = {col.Wave_number: float, col.Absorbance: float}
        return self



from typing import Any, Tuple

import pandas as pd

from .csv_config import CSVConfig
from .models import Sample


class DMASample(Sample):
    temp_range: Tuple[float, float] | None = None

    def model_post_init(
        self, __context: Any
    ) -> None:  # インスタンス生成後に実行される。csvから試料の大きさを取得する
        super().model_post_init(__context)
        if not self.temp_range:
            self.set_temp_range()
        return

    def set_temp_range(self) -> None:
        df = pd.read_csv(
            self.file_path,
            **CSVConfig(
                skiprows=[n for n in range(9)],
                nrows=1,
                usecols=[23, 31],
            ).to_dict(),
        )

        self.temp_range = df.values[0]
        return

    def get_result_df(self) -> pd.DataFrame:
        df = pd.read_csv(
            self.file_path,
            **CSVConfig().DMA().to_dict(),
        )
        return df
